<?php
/*
	http_client.php - librería cliente HTTP
	Copyright (C) 2008  Avelino Herrera Morales
	                    M.H.P. Servicios de Control S.L.

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

	class http_response {
		var $status = null;
		var $header = null;
		var $data = null;
	}

	function http_get_net_value($value) {
		$ret = "";
		for ($i = 0; $i < strlen($value); $i++) {
			$c = $value[$i];
			if (ereg("[0-9a-zA-Z]", $c))
				$ret .= $c;
			else if ($c == " ")
				$ret .= "+";
			else
				$ret .= "%".sprintf("%02x", ord($c));
		}
		return $ret;
	}

	function http_read_response($socket, $targetFile) {
		$ret = new http_response();
		$statusLine = "";
		$header = array();
		$data = array();
		$rawData = array();
		$STATUS_FIRST_LINE = 0;
		$STATUS_HEADER_VARIABLE_NAME = 1;
		$STATUS_HEADER_VARIABLE_VALUE = 2;
		$STATUS_PRE_DATA = 3;
		$STATUS_DATA = 4;
		$status = $STATUS_FIRST_LINE;
		$currentHeaderVariableName = "";
		$currentHeaderVariableValue = "";
		$targetFileHandle = null;
		if (($targetFile != null) && ($targetFile != ""))
			$targetFileHandle = fopen($targetFile, "w");
		while ($salida = socket_read($socket, 2048, PHP_BINARY_READ)) {
			for ($i = 0; $i < strlen($salida); $i++) {
				$c = $salida[$i];
				$rawData[] = $c;
				if ($status == $STATUS_FIRST_LINE) {
					if (($c != "\r") && ($c != "\n"))
						$statusLine .= $c;
					if ($c == "\n") {
						$status = $STATUS_HEADER_VARIABLE_NAME;
						$currentHeaderVariableName = "";
						$currentHeaderVariableValue = "";
					}
				}
				else if ($status == $STATUS_HEADER_VARIABLE_NAME) {
					if ($c == "\r")
						$status = $STATUS_PRE_DATA;
					else if ($c == ":")
						$status = $STATUS_HEADER_VARIABLE_VALUE;
					else
						$currentHeaderVariableName .= $c;
				}
				else if ($status == $STATUS_HEADER_VARIABLE_VALUE) {
					if (($c != "\r") && ($c != "\n"))
						$currentHeaderVariableValue .= $c;
					else if ($c == "\n") {
						$status = $STATUS_HEADER_VARIABLE_NAME;
						// echo "VARIABLE:  name='".$currentHeaderVariableName."' value='".trim($currentHeaderVariableValue)."'\n";
						$header[$currentHeaderVariableName] = trim($currentHeaderVariableValue);
						$currentHeaderVariableName = "";
						$currentHeaderVariableValue = "";
					}
				}
				else if ($status == $STATUS_PRE_DATA) {
					if ($c == "\n")
						$status = $STATUS_DATA;
				}
				else if ($status == $STATUS_DATA) {
					if ($targetFileHandle == null)
						$data[] = $c;
					else
						fwrite($targetFileHandle, $c);
				}
			}
		}
		//print_r($header);
		//echo "DATA: '".$data."'\n\n\n";
		//echo "RAWDATA: '".$rawData."'\n";
		$ret->header = $header;
		if ($targetFileHandle == null)
			$ret->data = $data;
		else {
			$ret->data = null;
			fclose($targetFileHandle);
		}
		$aux = strtok($statusLine, " ");
		if ($aux != false)
			$ret->status = strtok(" ");
		return $ret;
	}

	function http_get_net_values($variables) {
		$ret = "";
		$variableNames = array_keys($variables);
		for ($i = 0; $i < count($variableNames); $i++) {
			$vName = $variableNames[$i];
			$vValue = $variables[$vName];
			if ($i > 0)
				$ret .= "&";
			$ret .= $vName."=".http_get_net_value($vValue);
		}
		return $ret;
	}

	function http_get($host, $port, $resource, $targetFile) {
		$address = gethostbyname($host);
		$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		if ($socket < 0)
			return null;
		$result = socket_connect($socket, $address, $port);
		if (func_num_args() > 4) {
			$variables = func_get_arg(4);
			$resource .= "?".http_get_net_values($variables);
		}
		$input = "GET ".$resource." HTTP/1.1\r\nHost: ".$host."\r\n\r\n";
		//echo "\n......".$input."......\n";
		socket_write($socket, $input, strlen($input));
		$response = http_read_response($socket, $targetFile);
		socket_close($socket);
		return $response;
	}

	function http_post($host, $port, $resource, $targetFile) {
		$address = gethostbyname($host);
		$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		if ($socket < 0)
			return null;
		$result = socket_connect($socket, $address, $port);
		$postData = "";
		if (func_num_args() > 4) {
			$variables = func_get_arg(4);
			$postData = http_get_net_values($variables);
		}
		$input = "POST ".$resource." HTTP/1.1\r\nHost: ".$host."\r\nContent-Type: application/x-www-form-urlencoded\r\nContent-Length: ".strlen($postData)."\r\n\r\n".$postData;
		//echo "\n......".$input."......\n";
		socket_write($socket, $input, strlen($input));
		$response = http_read_response($socket, $targetFile);
		socket_close($socket);
		return $response;
	}

?> 
